#include <gtk/gtk.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glx.h>
#include <pthread.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include "prog.h"
#include <xmms/plugin.h>
#include <xmms/configfile.h>
#include "interface.h"
#include "callbacks.h"
#include "support.h"

static GtkWidget *config_dialog = NULL;
static GtkWidget *filesel_dialog = NULL;

static GtkWidget *which_entrybox = NULL;
static struct config initial_configuration;
static int which_filesel = 0;

struct config *gkos_config = NULL;

void gkos_read_config(void)
{
    ConfigFile *cfg;
    gchar *filename;

    filename = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);
    cfg = xmms_cfg_open_file(filename);

    if (cfg) {
	xmms_cfg_read_int(cfg, "Geekosphere", "num_lights", &gkos_config->num_lights);
	xmms_cfg_read_int(cfg, "Geekosphere", "detail_level", &gkos_config->detail_level);
	xmms_cfg_read_boolean(cfg, "Geekosphere", "lights_move", &gkos_config->lights_move);
	xmms_cfg_read_boolean(cfg, "Geekosphere", "lights_change_color", &gkos_config->lights_change_color);
	xmms_cfg_read_boolean(cfg, "Geekosphere", "calc_fps", &gkos_config->calc_fps);
	xmms_cfg_read_boolean(cfg, "Geekosphere", "tfdx_mode", &gkos_config->tdfx_mode);
	xmms_cfg_read_int(cfg, "Geekosphere", "render_style", &gkos_config->render_style);
	xmms_cfg_read_float(cfg, "Geekosphere", "window_width", &gkos_config->winw);
	xmms_cfg_read_float(cfg, "Geekosphere", "window_height", &gkos_config->winh);
	xmms_cfg_read_float(cfg, "Geekosphere", "max_fps", &gkos_config->max_fps);
	xmms_cfg_read_string(cfg, "Geekosphere", "texmap_filename", &gkos_config->texmap_filename);
	xmms_cfg_free(cfg);
    }
    g_free(filename);
}

void gkos_write_config(void)
{
    ConfigFile *cfg;
    gchar *filename;

    filename = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);
    cfg = xmms_cfg_open_file(filename);

    if (!cfg)
	cfg = xmms_cfg_new();

    xmms_cfg_write_int(cfg, "Geekosphere", "num_lights", gkos_config->num_lights);
    xmms_cfg_write_int(cfg, "Geekosphere", "detail_level", gkos_config->detail_level);
    xmms_cfg_write_boolean(cfg, "Geekosphere", "lights_move", gkos_config->lights_move);
    xmms_cfg_write_boolean(cfg, "Geekosphere", "lights_change_color", gkos_config->lights_change_color);
    xmms_cfg_write_boolean(cfg, "Geekosphere", "calc_fps", gkos_config->calc_fps);
    xmms_cfg_write_boolean(cfg, "Geekosphere", "tfdx_mode", gkos_config->tdfx_mode);
    xmms_cfg_write_int(cfg, "Geekosphere", "render_style", gkos_config->render_style);
    xmms_cfg_write_float(cfg, "Geekosphere", "window_width", gkos_config->winw);
    xmms_cfg_write_float(cfg, "Geekosphere", "window_height", gkos_config->winh);
    xmms_cfg_write_float(cfg, "Geekosphere", "max_fps", gkos_config->max_fps);
    xmms_cfg_write_string(cfg, "Geekosphere", "texmap_filename", gkos_config->texmap_filename);

    xmms_cfg_write_file(cfg, filename);
    xmms_cfg_free(cfg);
    g_free(filename);
}

void gkos_init_configuration(void)
{
    GLenum texture;

    if (gkos_config == NULL) {
	gkos_config = (struct config *) malloc(sizeof(struct config));
	gkos_config->num_lights = INIT_NUM_LIGHTS;
	gkos_config->detail_level = INIT_DETAIL;
	gkos_config->lights_move = INIT_LIGHTS_MOVE;
	gkos_config->lights_change_color = INIT_LIGHTS_CHANGE_COLOR;
	gkos_config->calc_fps = INIT_CALC_FPS;
	gkos_config->tdfx_mode = INIT_TDFX_MODE;
	gkos_config->render_style = INIT_RENDER_STYLE;
	gkos_config->texmap_filename = g_strdup_printf("%s/%s", DATADIR, DEFAULT_TEXMAP);
	gkos_config->max_fps = INIT_MAX_FPS;
	gkos_config->winw = INIT_WINW;
	gkos_config->winh = INIT_WINH;
    }

    gkos_read_config();
}

void geekosphere_configure(void)
{
    GtkWidget *num_lights, *detail_level, *lights_move, *lights_change_color,
	*render_style, *texmap_filename, *calc_fps, *tdfx_mode, *max_fps;
    
    GtkAdjustment *num_lights_adj, *detail_level_adj, *max_fps_adj;

    if (gkos_config == NULL)
	gkos_init_configuration();

    memcpy(&initial_configuration, gkos_config, sizeof(struct config));

    if (config_dialog == NULL)
	config_dialog = create_config_dialog();

    num_lights = lookup_widget(config_dialog, "num_lights_scale");
    
    num_lights_adj = gtk_range_get_adjustment(GTK_RANGE(num_lights));
    gtk_adjustment_set_value(num_lights_adj, gkos_config->num_lights);
    gtk_range_set_adjustment(GTK_RANGE(num_lights), num_lights_adj);

    detail_level = lookup_widget(config_dialog, "object_detail_scale");

    detail_level_adj = gtk_range_get_adjustment(GTK_RANGE(detail_level));
    gtk_adjustment_set_value(detail_level_adj, gkos_config->detail_level);
    gtk_range_set_adjustment(GTK_RANGE(detail_level), detail_level_adj);
    
    max_fps = lookup_widget(config_dialog, "max_fps_scale");

    max_fps_adj = gtk_range_get_adjustment(GTK_RANGE(max_fps));
    gtk_adjustment_set_value(max_fps_adj, gkos_config->max_fps);
    gtk_range_set_adjustment(GTK_RANGE(max_fps), max_fps_adj);

    lights_move = lookup_widget(config_dialog, "moving_lights_toggle");
    
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lights_move), 
				 gkos_config->lights_move);

    calc_fps = lookup_widget(config_dialog, "fps_toggle");
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(calc_fps), 
				 gkos_config->calc_fps);
    
    lights_change_color = lookup_widget(config_dialog, "light_color_change_toggle");
    
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lights_change_color),
				 gkos_config->lights_change_color);

    tdfx_mode = lookup_widget(config_dialog, "tdfx_mode");

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tdfx_mode),
				 gkos_config->tdfx_mode);

    switch(gkos_config->render_style) {
    case WIREFRAME_RENDER:
	render_style = lookup_widget(config_dialog, "wireframe_radio");
	break;
    case SOLID_RENDER:
	render_style = lookup_widget(config_dialog, "solid_radio");
	break;
    case TEXMAP_RENDER:
	render_style = lookup_widget(config_dialog, "texture_radio");
	break;
    default:
	render_style = lookup_widget(config_dialog, "wireframe_radio");
	break;
    }

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(render_style), TRUE);

    texmap_filename = lookup_widget(config_dialog, "texture_map_entry");
    gtk_entry_set_text(GTK_ENTRY(texmap_filename), gkos_config->texmap_filename);

    gtk_widget_show(config_dialog);
}

void
texture_map_filebox_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
    if (filesel_dialog == NULL)
	filesel_dialog = create_texture_map_filebox();

    which_filesel = TEXMAP_FILESEL;
    which_entrybox = lookup_widget(config_dialog, "texture_map_entry");
    gtk_widget_show(filesel_dialog);
    gdk_window_raise(filesel_dialog->window);
}

void apply_changes(void)
{
    GtkWidget *num_lights, *detail_level, *lights_move, *lights_change_color,
	*render_style, *texmap_filename, *calc_fps, *tdfx_mode, *max_fps;
    
    GtkAdjustment *num_lights_adj, *detail_level_adj, *max_fps_adj;

    num_lights = lookup_widget(config_dialog, "num_lights_scale");
    
    num_lights_adj = gtk_range_get_adjustment(GTK_RANGE(num_lights));
    gkos_config->num_lights = (gint) num_lights_adj->value;

    detail_level = lookup_widget(config_dialog, "object_detail_scale");

    detail_level_adj = gtk_range_get_adjustment(GTK_RANGE(detail_level));
    gkos_config->detail_level = detail_level_adj->value;

    max_fps = lookup_widget(config_dialog, "max_fps_scale");
    
    max_fps_adj = gtk_range_get_adjustment(GTK_RANGE(max_fps));
    gkos_config->max_fps = max_fps_adj->value;

    lights_move = lookup_widget(config_dialog, "moving_lights_toggle");

    gkos_config->lights_move = GTK_TOGGLE_BUTTON(lights_move)->active;
    
    lights_change_color = lookup_widget(config_dialog, "light_color_change_toggle");
    
    gkos_config->lights_change_color = GTK_TOGGLE_BUTTON(lights_change_color)->active;

    calc_fps = lookup_widget(config_dialog, "fps_toggle");
    gkos_config->calc_fps = GTK_TOGGLE_BUTTON(calc_fps)->active;

    tdfx_mode = lookup_widget(config_dialog, "tdfx_mode");
    gkos_config->tdfx_mode = GTK_TOGGLE_BUTTON(tdfx_mode)->active;

    render_style = lookup_widget(config_dialog, "wireframe_radio");
    if (GTK_TOGGLE_BUTTON(render_style)->active)
	gkos_config->render_style = WIREFRAME_RENDER;

    render_style = lookup_widget(config_dialog, "solid_radio");
    if (GTK_TOGGLE_BUTTON(render_style)->active)
	gkos_config->render_style = SOLID_RENDER;

    render_style = lookup_widget(config_dialog, "texture_radio");
    if (GTK_TOGGLE_BUTTON(render_style)->active)
	gkos_config->render_style = TEXMAP_RENDER;

    texmap_filename = lookup_widget(config_dialog, "texture_map_entry");
    gkos_config->texmap_filename = gtk_editable_get_chars(GTK_EDITABLE(texmap_filename), 0, -1);

    gkos_reset_all_lights();
    gkos_reset_all_objects();
}

void
config_apply_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
    apply_changes();
}

void
config_ok_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
    apply_changes();
    gkos_write_config();
    gtk_widget_hide(config_dialog);
}


void
config_cancel_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
    memcpy(gkos_config, &initial_configuration, sizeof(struct config));

    gkos_reset_all_lights();
    gkos_reset_all_objects();

    gtk_widget_hide(config_dialog);
}

void
on_filesel_ok_button_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
    gchar *filename;
    
    filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel_dialog));

    which_filesel = 0;

    gtk_entry_set_text(GTK_ENTRY(which_entrybox), filename);

    gtk_widget_hide(filesel_dialog);
}


void
on_filesel_cancel_button_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
    gtk_widget_hide(filesel_dialog);
}

gboolean
close_config_window                    (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
    memcpy(gkos_config, &initial_configuration, sizeof(struct config));
    gkos_reset_all_lights();
    gkos_reset_all_objects();
    gtk_widget_hide(config_dialog);
    return TRUE;
}


gboolean
close_filesel_window                   (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
    gtk_widget_hide(filesel_dialog);
    return TRUE;
}
